// =================================================================================================
//
//   File....... jm_Animation.cpp
//               -- adapted by David Betz from dc22_badge_master.spin
//               -- for internal use only!
//               -- no changes unless approved/reviewed by JonnyMac/Ryan
// 
//   Authors.... Jon "JonnyMac" McPhalen and Ryan "1o57" Clarke
//               Converted to C++ by David Betz
//               MIT License
//               -- see below for terms of use
//
//   E-mail..... jon@jonmcphalen.com
//               1o57@10000100001.org
//               dbetz@xlisper.com 
//
// =================================================================================================

#include <propeller.h>
#include "jm_Animation.h"

/* --------------- */
/*  L E D   F U N  */
/* --------------- */

// external function from dc22_badge_master.cpp
// this really needs to be factored better!
void pause(int ms);

jm_Animation::jm_Animation(int firstLed, int count)
{
  shift = firstLed;
  mask = ((1 << count) - 1) << firstLed;
  set_leds(0);
  cogId = -1;
}

int jm_Animation::start(uint8_t *p_tablex, int cyclesx)
{

// Start animation in background cog
// -- allows LED animation while doing other processes
// -- p_table is pointer (address of) animation table
// -- set cycles to 0 to run without stopping

  stop();
  
  p_table = p_tablex;
  cycles = cyclesx;
  cogId = cogstart(run_main, this, stack, sizeof(stack));
  
  return cogId;                                                 // return cog used
}

void jm_Animation::stop()
{
// Stop animation if currently running

  if (cogId >= 0)                                               // if running
    cogstop(cogId);                                             // stop the cog
    cogId = -1;                                                 // mark stopped 
}

void jm_Animation::run_main(void *par)
{
  ((jm_Animation *)par)->run();
}

void jm_Animation::run()
{
  uint8_t *p_leds;
  
// Run animation
// -- p_table is pointer (address of) animation table
// -- cycles is number of iterations to run
//    * 0 cycles runs "forever"
// -- usually called with animation.start()

  if (cycles <= 0)
    cycles = posx;                                              // run "forever"

  while (--cycles >= 0) {
    int count;
    p_leds = p_table;                                           // point to table
    count = *p_leds++;                                          // get steps in table
    while (--count >= 0) {                                      // repeat for steps in table
      set_leds(*p_leds++);                                      // update leds
      pause(*p_leds++);                                         // hold
    }
  }
  
  cogstop(cogid());                                             // stop this cog
  cogId = -1;                                                   // mark stopped
}

void jm_Animation::set_leds(int pattern)
{
  OUTA = (OUTA & ~mask) |  (pattern << shift);                  // write pattern to LEDs
  DIRA |= mask;                                                 // make LED pins outputs
}

/*

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/