// =================================================================================================
//
//   File....... jm_sircs_rx.cpp
//   Purpose.... SIRCS-compatible IR receiver
//   Author..... Jon "JonnyMac" McPhalen
//               Copyright (c) 2009-2014 Jon McPhalen  
//               -- see below for terms of use
//   E-mail..... jon@jonmcphalen.com
//   Started.... 
//   Updated.... 08 JUL 2014
//
// =================================================================================================

/*

    Example IR Connection (PNA4602M)
 
            ┌───┐
            │(*)│ 5v
            └┬┬┬┘ 
       ir ──┘│└──┘
              

    Protocol Reference:
    -- http://www.sbprojects.com/knowledge/ir/sirc.php
             
*/         

#include <propeller.h>
#include "jm_SircsRx.h"

int jm_SircsRx::start(int pin)
{
    use_cog_driver(jm_sircs_rx);

    stop();
    
    ircode = pin;
    irbits = _clkfreq / 1000000 * 2160;
    
    cogId = load_cog_driver(jm_sircs_rx, this);
    
    while (irbits >= 0)
        ;
    
    return cogId;
}

void jm_SircsRx::stop()
{

// Stops SIRCS receiver cog if running

  if (cogId >= 0) {
    cogstop(cogId);
    cogId = 0;
  }

  disable();
}

void jm_SircsRx::enable()
{

// Enables SIRCS receive process

  ircode = irbits = 0;
}

void jm_SircsRx::disable()
{

// Disable SIRCS receive process

  ircode = irbits = -1;
}
     
int jm_SircsRx::rx()
{

// Enables and waits for ir input
// -- warning: blocks until IR code received!
// -- does not remove code/bits from buffer

  enable();                                                     // allow ir rx
  while (irbits <= 0)                                           // wait for code
    ;

  return ircode;
}

int jm_SircsRx::rxcheck()
{

// Returns code if available, -1 if none
// -- must have previously been enabled
// -- does not remove code/bits from buffer

  if (irbits > 0)                                               // if code ready
    return ircode;
  else
    return -1;  
}
 
int jm_SircsRx::bit_count()
{

// Returns bit count of last ir code
// -- check status before using
// -- if 0, no code is available or disabled

  if (irbits > 0)
    return irbits;
  else
    return 0;
}
