// =================================================================================================
//
//   File....... jm_sircs_rx.h
//   Purpose.... SIRCS-compatible IR receiver
//   Author..... Jon "JonnyMac" McPhalen
//               Copyright (c) 2009-2014 Jon McPhalen  
//               -- see below for terms of use
//   E-mail..... jon@jonmcphalen.com
//   Started.... 
//   Updated.... 08 JUL 2014
//
// =================================================================================================

/*

    Example IR Connection (PNA4602M)
 
            ┌───┐
            │(*)│ 5v
            └┬┬┬┘ 
       ir ──┘│└──┘
              

    Protocol Reference:
    -- http://www.sbprojects.com/knowledge/ir/sirc.php
             
*/         

#ifndef __JM_SIRCSRX_H__
#define __JM_SIRCSRX_H__

class jm_SircsRx {
public:
    jm_SircsRx() : cogId(-1) {}
    jm_SircsRx(int pin) : cogId(-1) { start(pin); }
    ~jm_SircsRx() { stop(); }
    int start(int pin);
    void stop();
    void enable();
    void disable();
    int rx();
    int rxcheck();
    int bit_count();
private:
    volatile int ircode;                                        // rx'd code
    volatile int irbits;                                        // bits in code
    int cogId;                                                  // cog id
};

#endif

/*

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
