// =================================================================================================
//
//   File....... jm_sircs_tx.h
//   Purpose.... SIRCS-compatible transmitter
//               -- works with active-high and active-low connections
//   Author..... Jon "JonnyMac" McPhalen
//               Copyright (c) 2009-13 Jon McPhalen
//               -- see below for terms of use
//   E-mail..... jon@jonmcphalen.com
//   Started.... 
//   Updated.... 01 MAY 2013
//
// =================================================================================================

/*

    Typical connections:

              IR
      3.3v ──────┐
                    │
    ir pin ──────┘
               33



    Parallax QuickStart HID Board uses active-high output to IR LED 

          33  IR
    P9 ───────┐
                   
                    

    Protocol Reference:
    -- http://www.sbprojects.com/knowledge/ir/sirc.php
              
*/

#ifndef __JM_SIRCSTX_H__
#define __JM_SIRCSTX_H__

class jm_SircsTx {
public:
    jm_SircsTx() : cogId(-1) {}
    ~jm_SircsTx() { stop(); }
    int start(int pin, int hz);
    int startx(int pin, int hz, int framems);
    void stop();
    void tx(int code, int bits, int frames);
    int busy();
private:
    volatile int irframes;                                      // frames ( > 0 is flag to tx )     
    volatile int ircode;                                        // code
    volatile int irbits;                                        // bits in ir code
    int irstart;                                                // ticks in start bit
    int cogId;                                                  // cog id
};

#endif

/*

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
